#!/usr/bin/env python


#------------------------------------------------------------------------------
#
# 1. DISCLAIMER
#    THE SOFTWARE AND/OR RELATED MATERIALS ARE PROVIDED "AS-IS" WITHOUT
#    WARRANTY OF ANY KIND INCLUDING ANY WARRANTIES OF PERFORMANCE OR
#    MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE OR PURPOSE (AS SET FORTH
#    IN UCC 2312-2313) OR FOR ANY PURPOSE WHATSOEVER, FOR THE LICENSED
#    PRODUCT, HOWEVER USED.
#
#    IN NO EVENT SHALL CALTECH/JPL BE LIABLE FOR ANY DAMAGES AND/OR COSTS,
#    INCLUDING BUT NOT LIMITED TO INCIDENTAL OR CONSEQUENTIAL DAMAGES OF ANY
#    KIND, INCLUDING ECONOMIC DAMAGE OR INJURY TO PROPERTY AND LOST PROFITS,
#    REGARDLESS OF WHETHER CIT/JPL SHALL BE ADVISED, HAVE REASON TO KNOW,
#    OR IN FACT SHALL KNOW OF THE POSSIBILITY.
#
#    USER BEARS ALL RISK RELATING TO QUALITY AND PERFORMANCE OF THE SOFTWARE
#    AND/OR RELATED MATERIALS.
#
# 2. The user shall comply with all applicable U.S. export control laws and
#    regulations.  To the extent that the software is subject to U.S. export
#    control laws and regulations, the user has the responsibility to obtain
#    export licenses or other export authority as may be required before
#    exporting such information to foreign countries or providing access to
#    foreign persons.
#
#------------------------------------------------------------------------------

#$I AirsDb.py v0.75 2002-06-11
#$I Richard A. Forrister <rickf@glacier.jpl.nasa.gov>
#$I 2002-11-12 modify about GBAD processing
# modified by QDN on Jan 9, 2003
# modified by QDN on Jan 31, 2003
# modified by QDN on May 13, 2003
# modified by QDN on Apr 27, 2005 : to support NCEP GRID files

import os, sys, getopt, string, airs_env, airs_utils, fnmatch, re, apid, meta
import pcf_template, esdt, glob, expnset, renameFiles, pcf, pathCheck, time, genMeta
import scanMET

METATEMPLATE = 'METATEMPLATE.met'

PKT_DIR=""
GBAD_LIST=['1']
L1B_INPUT_SETS={}
L1A_OUTPUT_LIDS = ['7111','7113','7120','7121','7122','7123','7130','7131','7140']
L1B_OUTPUT_LIDS = ['7210','7211','7212','7213','7241','7250','7251','7252','7253',
		   '7261','7262','7263','7264','7265','7266']
L2_INPUT_LIDS=['7210','7211','7212','7213']
opt_prev_L2_INPUT_LIDS=['6300','6302']
opt_next_L2_INPUT_LIDS=['6310','6312']
L2_INPUT_FILES={}
L2_OUTPUT_LIDS = ['7300', '7301', '7302', '7303', '7310', '7311']

GRIB_LIDS = [ '0, 2203,T21Z.gblav_PGrbF03',
              '1, 2206,T00Z.gblav_PGrbF06',
              '1, 2209,T03Z.gblav_PGrbF09',
              '1, 2213,T03Z.gblav_PGrbF03',
              '1, 2216,T06Z.gblav_PGrbF06',
              '1, 2219,T09Z.gblav_PGrbF09',
              '1, 2223,T09Z.gblav_PGrbF03',
              '1, 2226,T12Z.gblav_PGrbF06', 
              '1, 2229,T15Z.gblav_PGrbF09',
              '1, 2233,T15Z.gblav_PGrbF03', 
              '1, 2236,T18Z.gblav_PGrbF06', 
              '1, 2239,T21Z.gblav_PGrbF09',
              '1, 2243,T21Z.gblav_PGrbF03',
              '2, 2246,T00Z.gblav_PGrbF06', 
              '2, 2249,T03Z.gblav_PGrbF09']

GBAD_IDX = 0
GRIB_IDX = 0

ST_tm = 0.0
ET_tm = 0.0
ST_UTC = ""
ET_UTC = ""
first_gr = 0
last_gr  = 0
num_gr   = -1
debug_opt = 0
uncompress_opt = "Nominal" 
runl2mode_opt = -1
dir_utc_leap_opt = ""
ignore_opt = 0
remove_opt = 0
print_opt = 0
pcf_opt = 0
proc_opt = "1b" # by default we want to execute only L1A and L1B (not L2).
genMET_opt = 0
Override_ST_Date = ""
Override_ST_Granule = -1
Override_Num_Granule = -1
curr_day = []

####################################################
def printDEBUG(prt_str):
   #global debug_opt
   if (debug_opt == 1):
      print "DEBUG_MSG:%s" % prt_str

####################################################
def toDate(TaiValue):
   Result_UTC = os.popen('run_TAIandUTC %f'%TaiValue).readline()[:-1]
   Result_Date = (Result_UTC[0:10]).replace('-','.')
   return Result_Date

####################################################
def toUTC(TaiValue):
   Result_UTC = os.popen('run_TAIandUTC %f'%TaiValue).readline()[:-1]
   return Result_UTC

####################################################
def toTAI(UTCValue):
   Result_TAI = float(os.popen('run_TAIandUTC %s'%UTCValue).readline()[:-1])
   return Result_TAI

####################################################
def FirstGranuleTAI(DATE):
   TAI_sec_ref = toTAI('2001-09-16T20:05:26.000000Z')
   TAI_date = toTAI(DATE+'T00:00:00.000000Z')
   TAI_first = TAI_sec_ref+int((TAI_date - TAI_sec_ref + 359.999)/360.0)*360.0
   return TAI_first

######################################################
def GranuletoST_Date_IGranule(Date,IGranule):
   TAI_start = FirstGranuleTAI(Date)+(IGranule-1)*360.0
   return toUTC(TAI_start)

######################################################
def GranuletoET_Date_IGranule(Date,IGranule):
   TAI_end = FirstGranuleTAI(Date)+(IGranule-1)*360.0+359.999999
   return toUTC(TAI_end)

######################################################
def GranuleST_UTC(time_UTC):
   time_TAI = toTAI(time_UTC)
   time_TAI_ref = toTAI('2001-09-16T20:05:26.000000Z')
   ST_TAI = int((time_TAI - time_TAI_ref)/360.0)*360.0+time_TAI_ref
   ST_UTC = toUTC(ST_TAI)
   return ST_UTC

#####################################################
def GranuleET_UTC(time_UTC):
   time_TAI = toTAI(time_UTC)
   time_TAI_ref = toTAI('2001-09-16T20:05:26.000000Z')
   ET_TAI = int((time_TAI - time_TAI_ref)/360.0)*360.0+time_TAI_ref+359.999999
   ET_UTC = toUTC(ET_TAI)
   return ET_UTC

#####################################################
def IGranuleST_UTC(time_UTC):
   ET_TAI = toTAI(GranuleET_UTC(time_UTC[:10]+'T00:00:00.000000Z'))
   curr_TAI = toTAI(time_UTC)
   if ( curr_TAI < ET_TAI) :
      ET_TAI = ET_TAI - 24.0*60.0*60.0
      ET_TAI = toTAI(GranuleET_UTC(toUTC(ET_TAI-0.5)))
   IGranule = int((curr_TAI-ET_TAI+360.0)/360.0)
   IDate = (toUTC(ET_TAI))[:10]
   return [IDate,IGranule]

####################################################
def time_stamp():
   tmTpl = time.gmtime(time.time())
   str = "%02d%03d%02d%02d%02d" % \
         (tmTpl[0]%100, tmTpl[7], tmTpl[3], tmTpl[4], tmTpl[5])
   return str

#####################################################
def getDateStr():
   tmTpl = time.localtime(time.time())
   str = "%04d%03d-%02d%02d%02d" % \
         (tmTpl[0], tmTpl[7], tmTpl[3], tmTpl[4], tmTpl[5])
   return str

#####################################################
def procGBAD(gbadName):
   global GBAD_IDX
   global TimeStamp
   WRK_DIR = airs_env.getENV('WORKING_DIR')
   if (os.environ.has_key('GBAD_DATA')):
      DATA_DIR = os.environ['GBAD_DATA']
   else:
      DATA_DIR = os.path.join(airs_env.getENV('AIRSDB'),'gbad/localdata')
   BASE_NAME =  os.path.splitext(os.path.basename(gbadName))[0][22:]
   Time_str = time_stamp()
   Time_str = TimeStamp

   #Generate the GBAD (aqua_main) command line & execute
   #This requires writing a configuration file for GBAD to the
   #working directory, and then executing the command os.popen

   Attitude_file = '%s/PM1ATTR_%s%s'%(WRK_DIR,Time_str,BASE_NAME)
   Ephemeris_file = '%s/PM1EPHND_%s%s'%(WRK_DIR,Time_str,BASE_NAME)
   norad_file = '%s/%s'%(DATA_DIR,'noradfile')
   #Config_file = 'config-gbad-%s-%d.log'%(time_stamp(), GBAD_IDX)
   Config_file = 'config-gbad-%s-%d.log'%(Time_str, GBAD_IDX)
   GBAD_IDX = GBAD_IDX+1

   # Create the configure file for GBAD software (aqua_main)
   CF=open(os.path.join(WRK_DIR,Config_file),'w')
   CF.write('-%s\t%s\n' % ('packetfile',gbadName))
   CF.write('-%s\t%s\n' % ('ephemerisfile',Ephemeris_file))
   CF.write('-%s\t%s\n' % ('attitudefile',Attitude_file))
   CF.write('-%s\t%s\n' % ('noradfile',norad_file))
   CF.write('-%s\t%s\n' % ('listconfig','yes'))
   CF.close()

   # Run the aqua_main program
   GBAD_CMD = 'aqua_main -configfile %s/%s' % (WRK_DIR,Config_file)
   RESULTS = os.popen(GBAD_CMD).readlines()

   # Will generate the .met file for these output attitude and
   # ephemeris files later.


#########################################################################
# gen_L0: Check to see if there are PDS/EDS files in AIRSDB_PACKETS.
# If yes, then create links for each entry in WORKING_DIR, else use the
# external to generate headers and PDS data in WORKING_DIR.  Also set
# START_TIME and END_TIME in airs_env module.
#########################################################################
def genL0():
   global ST_tm,ET_tm,ST_UTC,ET_UTC,first_gr,last_gr,num_gr
   global Override_ST_Date,Override_ST_Granule,Override_Num_Granule
   global GRIB_IDX,curr_day

   ST={'AIRS':0.0,
       'AMSU':0.0,
       'HSB':0.0}
   ET={'AIRS':0.0,
       'AMSU':0.0,
       'HSB':0.0}

   INST_GROUPS = []
   USED_APIDS = []

   fncName='genL0'

   CURR_DIR = os.getcwd()
   WRK_DIR = airs_env.getENV('WORKING_DIR')
   os.chdir(WRK_DIR)
   PKT_DIR = os.environ['AIRSDB_PACKETS']

   # For GBAD data sets
   # If the PM1ATT* and PM1EPH* files exist, use them
   # otherwise, if there is P1540957...1.PDS, run gbad package
   FileList = os.listdir(PKT_DIR)
   ATT_EPH_Pattern = re.compile("^PM1[AE]")
   ATT_EPH_FileList = filter(ATT_EPH_Pattern.search,FileList)
   GBAD_Pattern = re.compile("^[PE]15409+(A*\d)+1.[PE]DS$")
   GBAD_FileList = filter(GBAD_Pattern.search,FileList)
   if(len(ATT_EPH_FileList) > 0 ):
      for file in ATT_EPH_FileList:
         if(os.path.exists(os.path.join(PKT_DIR,file))):
            os.symlink(os.path.join(PKT_DIR,file),os.path.join(WRK_DIR,file))
            print "Info: link %s to working directory"%file
            airs_utils.logstr('genL0','link %s to working directory'%file)
            printDEBUG("link %s to working directory"%file)
   elif(len(GBAD_FileList) > 0):
      for file in GBAD_FileList:
         if(os.path.exists(os.path.join(PKT_DIR,file))):
            os.symlink(os.path.join(PKT_DIR,file),os.path.join(WRK_DIR,file))
            print "Info: link and process GBAD %s in working directory"%file
            airs_utils.logstr('genL0',\
                   'GBAD file: %s is used to generate the att+eph file'%file)
            printDEBUG(\
                   "GBAD file: %s is used to generate the att+eph file"%file)
            procGBAD(os.path.join(WRK_DIR,file))
  
   # Check for the .met files for the Att and Eph files
   gen_met_att_eph(WRK_DIR)

   # For GRIB data sets
   # If the NCEP* files exist, use them by first making a logical link
   FileList = os.listdir(PKT_DIR)
   NCEP_Pattern = re.compile("^NCEP")
   NCEP_FileList = filter(NCEP_Pattern.search,FileList)
   if(len(NCEP_FileList) > 0 ):
      for file in NCEP_FileList:
         if(os.path.exists(os.path.join(PKT_DIR,file))):
            GRIB_IDX = GRIB_IDX + 1
            os.symlink(os.path.join(PKT_DIR,file),os.path.join(WRK_DIR,file))
            print "Info: link %s to working directory"%file
            airs_utils.logstr('genL0','link %s to working directory'%file)
            printDEBUG("link %s to working directory"%file)


   # For level 0 data sets
   FileList = os.listdir(PKT_DIR)
   L0_Pattern = re.compile("^[PE]1540[234]+(A*\d)+1.[PE]DS$")
   FileList = filter(L0_Pattern.search,FileList)
   printDEBUG ("Packet dir %s" % PKT_DIR)
   for file in FileList:
      if(os.path.exists(os.path.join(PKT_DIR,file))):
         airs_utils.logstr('genL0','link %s to working directory'%file)
         print "Info: link %s to working directory" % file
         printDEBUG("link %s to working directory"%file)
         os.symlink(os.path.join(PKT_DIR,file),os.path.join(WRK_DIR,file))
      cr_file = re.sub("1.PDS$","0.PDS",file)
      cr_file = re.sub("1.EDS$","0.EDS",cr_file)
      met_file = re.sub("1.PDS$","1.PDS.met",file)
      met_file = re.sub("1.EDS$","1.EDS.met",met_file)
      wrk_data_file = os.path.join(WRK_DIR,file)
      wrk_cr_file = os.path.join(WRK_DIR,cr_file)
      wrk_met_file = os.path.join(WRK_DIR,met_file)
      if(os.path.exists(os.path.join(PKT_DIR,cr_file)) and
         os.path.exists(os.path.join(PKT_DIR,met_file)) and
         genMET_opt == 0):
         airs_utils.logstr('genL0',\
                  'link to existing construction and met files')
         print "Info: Link to existing construction and met files"
         if(os.path.exists(os.path.join(PKT_DIR,cr_file))):
            printDEBUG("link to existing construction file")
            os.symlink(os.path.join(PKT_DIR,cr_file),wrk_cr_file)
         if(os.path.exists(os.path.join(PKT_DIR,met_file))):
            printDEBUG("link to existing met file")
            os.symlink(os.path.join(PKT_DIR,met_file),wrk_met_file)
      else:
         airs_utils.logstr('genL0',\
                  'Generating the construction file and met files %s'\
                   %WRK_DIR)
         print "Info: Generate the construction file + met at %s"\
                   %WRK_DIR
         printDEBUG('Run the program to generate the construction file+met%s'\
                   %WRK_DIR)
         # Run external program to generate the construction and met files
         CR_CMD = 'run_edos4 %s' % (file)
         RESULT_CR = os.popen(CR_CMD).readlines()
         
      # Scan the met file for starting and ending time
      if(os.path.exists(wrk_met_file)):
         printDEBUG('The met file is located at %s'%wrk_met_file)
         F_met = scanMET.scanODL(wrk_met_file)
         tAPID = apid.extractMetAPID(file)
         Inst = apid.getProcGroup(tAPID)
         if not (tAPID in USED_APIDS):
             USED_APIDS.append(tAPID)
         starttime=toTAI(F_met['RangeBeginningDate']+'T'\
                           +F_met['RangeBeginningTime'])
         endtime=toTAI(F_met['RangeEndingDate']+'T'+F_met['RangeEndingTime'])
         if(ST[Inst] == 0.0):
            ST[Inst] = starttime
            ET[Inst] = endtime
         if(ST_tm == 0.0):
            ST_tm = starttime
            ET_tm = endtime
         ST[Inst] = min(ST[Inst],starttime)
         ET[Inst] = max(ET[Inst],endtime)
         ST_tm = min(ST_tm,starttime)
         ET_tm = max(ET_tm,endtime)
         sGran = (IGranuleST_UTC(toUTC(ST_tm)))[-1]
         eGran = (IGranuleST_UTC(toUTC(ET_tm)))[-1]
         printDEBUG('Filename : %s'%file)
         printDEBUG('Time: %s, %s'%(toUTC(starttime),toUTC(endtime)))
         printDEBUG('Time: %f, %f'%(starttime,endtime))
         printDEBUG('Granule: %d, %d'%(sGran,eGran))
      else:
         printDEBUG('Missing the met file at %s'%wrk_met_file)
   for tAPID in USED_APIDS:
       Inst = apid.getInstrumentName(tAPID)
       if not (Inst in INST_GROUPS):
          INST_GROUPS.append(Inst)

   # Jan 9,2003 by QDN: modified ST_tm and ET_tm to boundary of granule
   if(Override_ST_Date != "" and Override_ST_Granule != -1 \
      and Override_Num_Granule != -1):
       ST_tm = toTAI(GranuletoST_Date_IGranule(Override_ST_Date,\
                     Override_ST_Granule))
       ET_tm = ST_tm + (Override_Num_Granule)*360.0  - 180.
       printDEBUG('(genL0): Orverride value of Granule = %d with Num=%d'%
                    (Override_ST_Granule,Override_Num_Granule))

   # Calculate the start time and number of granules
   first_gr = (IGranuleST_UTC(toUTC(ST_tm)))[-1]
   ST_tm = toTAI(GranuleST_UTC(toUTC(ST_tm)))
   ST_UTC = toUTC(ST_tm)
   last_gr = (IGranuleST_UTC(toUTC(ET_tm)))[-1]
   ET_tm = toTAI(GranuleET_UTC(toUTC(ET_tm)))
   ET_UTC = toUTC(ET_tm)
   num_gr = int((ET_tm - ST_tm + 359.0)/360.0)

   # Calculate the previous day, the curr day and the next day
   curr_day.append( toDate(ST_tm-24*60*60) )
   curr_day.append( toDate(ST_tm) )
   curr_day.append( toDate(ST_tm+24*60*60) )

   # Print out the information about the output granules
   printDEBUG ('(genL0): Start UTC = %s , Start time = %f'%(ST_UTC,ST_tm))
   printDEBUG ('(genL0): End   UTC = %s , End   time = %f'%(ET_UTC,ET_tm))
   printDEBUG ('(genL0): Start Granule = %d, endGranule = %d, num_gr = %d' % \
               (first_gr,last_gr,num_gr))

   print "Info: Start UTC = %s , Start tm=%f"%(ST_UTC,ST_tm)
   print "Info: End   UTC = %s , End   tm=%f"%(ET_UTC,ET_tm)
   print "Info: Start Granule = %d, endGranule = %d, num_gr = %d" % \
               (first_gr,last_gr,num_gr)
   print "Info: Processing output product level = %s" % proc_opt

   for Inst in INST_GROUPS:
      StStr = 'STARTTIME-%s' % (Inst)
      EtStr = 'ENDTIME-%s' % (Inst)
      airs_utils.logstr(fncName, \
            '<%s> - <%f> - <%s> - <%s>' % \
            (StStr, ST[Inst], toUTC(ST[Inst]), ST_UTC))
      airs_utils.logstr(fncName, \
            '<%s> - <%f> - <%s> - <%s>' % \
            (EtStr, ET[Inst], toUTC(ET[Inst]), ET_UTC))
      airs_env.setENV(StStr, ST_UTC)
      airs_env.setENV(EtStr, ET_UTC)
      printDEBUG ( '(genL0) Inst= %s ,Start = %s, End = %s' %\
                   (Inst,toUTC(ST[Inst]),toUTC(ET[Inst])))
      printDEBUG ( '(genL0) Inst= %s ,RStart= %s, REnd = %s'%\
                   (Inst,airs_env.getENV(StStr),airs_env.getENV(EtStr)))

   os.chdir(CURR_DIR)
   return INST_GROUPS


####################################################################

#########################################################################
def metaCheck(fName):
   NPARTS = os.path.split(fName)
   fPath = NPARTS[0]
   Name = NPARTS[1]
   PARTS = os.path.splitext(Name)
   Name = PARTS[0]
   Ext = PARTS[1]
   ChkChr = Name[(len(Name)-1)]
   if ChkChr == '0':  # Check for a matching metadata file to the "1."
      Name=Name[:-1]+'1'
      basename = Name+Ext
      FullName = os.path.join(fPath, basename)
      MetaName = FullName+'.met'
      CK = glob.glob(MetaName)
      if (len(CK) == 0):  # No metadata file exists, generate it.
         DataSet = [fName, FullName]
	 genMeta.genMet(DataSet)
	 # Need to confirm that output metadata file is created in correct
	 # directory - probably WRKDIR.
         #print 'metaCheck: generate met file at %s' % airs_env.getENV('WORKING_DIR')
	 genMeta.genMetadataFile(METATEMPLATE, airs_env.getENV('WORKING_DIR'))

#########################################################################
def genLIDLine(LID, TBL, DIR, FName):
   Str = TBL.getPCFLine(LID)
   if (Str[0] == '#'):
      Str = Str[1:]
      pcfLn = pcf.PCF_LINE(Str)
      pcfLn.repFld(1,FName)
      pcfLn.repFld(2,DIR)
      pcfLn.repFld(5,FName+'.met')
      pcfLn.repFld(4, os.path.join(DIR, FName))
      Str = '%s'%(pcfLn)
      TBL.repLine(LID, Str)
   airs_utils.logstr('genLIDLine','%d->%s' % (LID, Str))

#########################################################################
def repLIDLine(LID, TBL, DIR, FName):
   Str = TBL.getPCFLine(LID)
   pcfLn = pcf.PCF_LINE(Str)
   pcfLn.repFld(1,FName)
   pcfLn.repFld(2,DIR)
   Str = '%s'%(pcfLn)
   TBL.repLine(LID, Str)
   airs_utils.logstr('repLIDLine','%d->%s' % (LID, Str))

#########################################################################
def proc_CO_Files(CARY_DIR='.'):
   CARY_DIR = PKT_DIR
   CL1 = glob.glob(os.path.join(CARY_DIR,'*carryout*'))
   CL2 = glob.glob(os.path.join(CARY_DIR,'*CARRYOUT*'))
   CARRYLIST = []
   TMP_LIST = CL1+CL2
   if (len(TMP_LIST) > 0):
      for item in TMP_LIST:
         Name = os.path.split(item)[1]
	 fExt = os.path.splitext(os.path.basename(item))[1]
	 if (fExt != '.met'):
	    CARRYLIST.append(Name)
   else:
      return
   # If the length of the CARRYLIST is > 2 ignore all entries after the
   # first two.  If == 1, then only process LID 4007.
   ListLen = len(CARRYLIST)
   if (ListLen >= 1):
      genLIDLine(4007, PCF_TBL, CARY_DIR, CARRYLIST[0])
   if (ListLen >= 2):
      genLIDLine(4008, PCF_TBL, CARY_DIR, CARRYLIST[1])

#########################################################################
def gen_met_att_eph(ATT_EPH_DIR):

   # Check if the .met files do not exist, create dummy ones
   ATT = os.listdir(ATT_EPH_DIR)
   ATT = filter(re.compile("^PM1ATT").search,ATT)
   EPH = os.listdir(ATT_EPH_DIR)
   EPH = filter(re.compile("^PM1EPH").search,EPH)
   for file in ATT:
      if ( file[-4:] != '.met'):
         if not (os.path.exists(os.path.join(ATT_EPH_DIR,file+'.met'))):
            GBAD_CMD_MET_ATT = 'dump_att.pl -m %s'%file
            RESULT_ATT = os.popen(GBAD_CMD_MET_ATT).readlines()
            print "Info: generate the met file for ATT file: %s"%file
            airs_utils.logstr('gen_met_att_eph',\
                              'generate met file for Attitude file %s'%file)
   for file in EPH:
      if ( file[-4:] != '.met'):
         if not (os.path.exists(os.path.join(ATT_EPH_DIR,file+'.met'))):
            GBAD_CMD_MET_EPH = 'dump_eph.pl -m %s'%file
            RESULT_EPH = os.popen(GBAD_CMD_MET_EPH).readlines()
            print "Info: generate the met file for EPH file: %s"%file
            airs_utils.logstr('gen_met_att_eph',\
                              'generate met file for Ephemeris file %s'%file)

#########################################################################
def getAtt_n_Ephem_Files(ATT_DIR='.'):
   fncName = 'getAtt_n_Ephem_Files'

   if (ATT_DIR == '.'):
      ATT_DIR = airs_env.ENV['WORKING_DIR']

   DELLIST=[]
   ATTLIST= glob.glob(os.path.join(ATT_DIR,'PM1ATT*'))
   EPHLIST= glob.glob(os.path.join(ATT_DIR,'PM1EPH*'))
   for x in range(len(ATTLIST)):
      ATTLIST[x] = string.strip(ATTLIST[x])
      ATTLIST[x] = os.path.basename(ATTLIST[x])
      fExt = os.path.splitext(os.path.basename(ATTLIST[x]))[1]
      if (fExt == '.met'):
         DELLIST.append(x)
   DELLIST.reverse()
   for x in DELLIST:
      del ATTLIST[x]

   DELLIST=[]
   for x in range(len(EPHLIST)):
      EPHLIST[x] = string.strip(EPHLIST[x])
      EPHLIST[x] = os.path.basename(EPHLIST[x])
      fExt = os.path.splitext(os.path.basename(EPHLIST[x]))[1]
      if (fExt == '.met'):
         DELLIST.append(x)
   DELLIST.reverse()
   for x in DELLIST:
      del EPHLIST[x]

   AttList = expnset.ExpSet(ATT_DIR, PCF_TBL.getPCFLine(10502), ATTLIST, 1)
   AttList.setPathFld(2)
   AttList.setMetaFld()
   AttList.setLGIDFld(4)
   EphList = expnset.ExpSet(ATT_DIR, PCF_TBL.getPCFLine(10501), EPHLIST, 1)
   EphList.setPathFld(2)
   EphList.setMetaFld()
   EphList.setLGIDFld(4)
   airs_utils.logstr(fncName, 'ATT_DIR = %s' % (ATT_DIR))
   airs_utils.loglist(fncName, ATTLIST, 'ATTLIST')
   airs_utils.loglist(fncName, EPHLIST, 'EPHLIST')
   return (AttList.getExpList(), EphList.getExpList())

#########################################################################
def getGRIB_Files():
   global  curr_day
   fncName = 'getGRIB_Files'

   GRIB_DIR = airs_env.ENV['WORKING_DIR']
   for x in GRIB_LIDS:
      Fields = string.split(x,",",3)
      GRIB_lid  = string.atoi(Fields[1])
      GRIB_Patt = 'NCEP.' + curr_day[string.atoi(Fields[0])] + '.' + Fields[2] + '*'

      DELLIST = []
      GRIBLIST = glob.glob(os.path.join(GRIB_DIR,GRIB_Patt))

      if( len(GRIBLIST) > 0 ): 
         for x in range(len(GRIBLIST)):
            print "Info: Found GRIB file ",os.path.basename(GRIBLIST[x])
            GRIBLIST[x] = string.strip(GRIBLIST[x])
            GRIBLIST[x] = os.path.basename(GRIBLIST[x])
            fExt = os.path.splitext(os.path.basename(GRIBLIST[x]))[1]
            if (fExt == '.met'):
               DELLIST.append(x)
         DELLIST.reverse()
         for x in DELLIST:
            del GRIBLIST[x]

         GribList = expnset.ExpSet(GRIB_DIR, L2_PCF.getPCFLine(GRIB_lid), 
                                   GRIBLIST, 1)
         GribList.setPathFld(2)
         GribList.setMetaFld()
         GribList.setLGIDFld(4)
         airs_utils.logstr(fncName, 'GRIB_DIR = %s' % (GRIB_DIR))
         airs_utils.loglist(fncName, GRIBLIST, 'GRIBLIST')

         L2_PCF.expandMultiList(GRIB_lid,GribList.getExpList())
      
   return 

#########################################################################
# Handles the generation of output names, LGID's, etc for proper expansion 
# of output sets.  Needs first granule, last granule values.
#########################################################################
def expandOutputEntries(NumGran, FirstGran, LastGran, TBL, LVL):
   FncName='expandOutputEntries'
   
   printDEBUG("expandOutputEntries  at lvl = %s"%LVL)
   if (LVL == 'L1A'):
      OUTPUT_LIDS = L1A_OUTPUT_LIDS
   elif (LVL == 'L1B'):
      OUTPUT_LIDS = L1B_OUTPUT_LIDS
   printDEBUG(": expandOutputEntries output_lids = %s"%OUTPUT_LIDS)

   airs_utils.loglist(FncName,OUTPUT_LIDS, 'Level = %s'%(LVL))
   for tLID in OUTPUT_LIDS:
      LID = string.atoi(tLID)
      PCF_Line = TBL.getPCFLine(LID)
      airs_utils.logstr(FncName, 'Looking for -%d-, got -%s'%(LID,PCF_Line))
      if (PCF_Line == ''):
         airs_utils.logstr(FncName, 'NO PCF_LINE HAD LID = -%s-'%(tLID))
         continue
      airs_utils.logstr(FncName, '-%s->%s-'%(tLID,PCF_Line))
      # Generate name list
      PARTS = string.split(PCF_Line, '|')
      NameList = []
      for x1 in range ( 0,string.atoi(NumGran)):
         x = (x1+string.atoi(FirstGran)-1)%240 + 1 
         NewName = '%s-%03d' % (PARTS[1], x)
         L2_INPUT_FILES[NewName]=1
         NameList.append(NewName)
      ExpnList = expnset.ExpSet(airs_env.ENV['WORKING_DIR'], TBL.getPCFLine(LID), NameList, 1)
      ExpnList.setPathFld(2)
      ExpnList.setMetaFld(1)
      TBL.expandMultiList(LID, ExpnList.getExpList())
      printDEBUG(" at LID=%d -> L1B output is %s"%(LID,NameList[0]))
      if (`LID` in apid.L1B_APIDS):
         L1B_INPUT_SETS[LID] = ExpnList.getExpList()

#########################################################################
# Handle the generation of L2 output names, LGIDS
# Need only one granule number
#########################################################################
def expandOutputL2(IGran,TBL):
   FncName='expandOutputL2'
   for tLID in L2_OUTPUT_LIDS:
      LID = string.atoi(tLID)
      PCF_Line = TBL.getPCFLine(LID)
      airs_utils.logstr(FncName,'Looking for -%d-, got %s'%(LID,PCF_Line))
      if(PCF_Line == ''):
         airs_utils.logstr(FncName,'NO PCF_LINE HAD LID = -%s-'%tLID)
         continue
      airs_utils.logstr(FncName,'-%s->%s'%(tLID,PCF_Line))
      PARTS = string.split(PCF_Line,'|')
      NameList = []
      NameList.append('%s-%03d'%(PARTS[1],IGran))
      ExpnList = expnset.ExpSet(airs_env.ENV['WORKING_DIR'],TBL.getPCFLine(LID),NameList,1)
      ExpnList.setPathFld(2)
      ExpnList.setMetaFld(1)
      TBL.expandMultiList(LID,ExpnList.getExpList())

#########################################################################
# Handle the input files for L2 products
# Need only one granule number
#########################################################################
def expandInputL2(IGran,TBL):
   FncName='expandInputL2'
   for tLID in L2_INPUT_LIDS:
      LID = string.atoi(tLID)
      PCF_Line = TBL.getPCFLine(LID)
      airs_utils.logstr(FncName,'Looking for -%d-, got %s'%(LID,PCF_Line))
      if (PCF_Line == ''):
         airs_utils.logstr(FncName,'NO PCF_LINE HAD LID = -%s-'%tLID)
         continue
      airs_utils.logstr(FncName,'-%s->%s'%(tLID,PCF_Line))
      PARTS = string.split(PCF_Line,'|')
      NameList = []
      NewName = '%s-%03d'%(PARTS[1],IGran)
      NameList.append(NewName)
      if ( L2_INPUT_FILES.has_key(NewName)) :
         ExpnList = expnset.ExpSet(airs_env.ENV['WORKING_DIR'],TBL.getPCFLine(LID),NameList,1)
         ExpnList.setPathFld(2)
         ExpnList.setMetaFld(1)
         TBL.expandMultiList(LID,ExpnList.getExpList())

   for tLID in opt_prev_L2_INPUT_LIDS:
      LID = string.atoi(tLID)
      PCF_Line = TBL.getPCFLine(LID)
      airs_utils.logstr(FncName,'Looking for -%d-, got %s'%(LID,PCF_Line))
      if (PCF_Line == ''):
         airs_utils.logstr(FncName,'NO PCF_LINE HAD LID = -%s-'%tLID)
         continue
      airs_utils.logstr(FncName,'-%s->%s'%(tLID,PCF_Line))
      PARTS = string.split(PCF_Line,'|')
      NameList = []
      NewName = '%s-%03d'%(PARTS[1],IGran-1)
      NameList.append(NewName)
      if ( L2_INPUT_FILES.has_key(NewName)) :
         ExpnList = expnset.ExpSet(airs_env.ENV['WORKING_DIR'],TBL.getPCFLine(LID),NameList,1)
         ExpnList.setPathFld(2)
   
         ExpnList.setMetaFld(1)
         TBL.expandMultiList(LID,ExpnList.getExpList())

   for tLID in opt_next_L2_INPUT_LIDS:
      LID = string.atoi(tLID)
      PCF_Line = TBL.getPCFLine(LID)
      airs_utils.logstr(FncName,'Looking for -%d-, got %s'%(LID,PCF_Line))
      if (PCF_Line == ''):
         airs_utils.logstr(FncName,'NO PCF_LINE HAD LID = -%s-'%tLID)
         continue
      airs_utils.logstr(FncName,'-%s->%s'%(tLID,PCF_Line))
      PARTS = string.split(PCF_Line,'|')
      NameList = []
      NewName = '%s-%03d'%(PARTS[1],IGran+1)
      NameList.append(NewName)
      if ( L2_INPUT_FILES.has_key(NewName)):
          ExpnList = expnset.ExpSet(airs_env.ENV['WORKING_DIR'],TBL.getPCFLine(LID)
,NameList,1)
          ExpnList.setPathFld(2)
          ExpnList.setMetaFld(1)
          TBL.expandMultiList(LID,ExpnList.getExpList())


#########################################################################
# L1A.ksh & pgeenv.ksh both appear to check the users original environ	#
# ment, and therefore loose the appropriate definitions of AIRSROOT and	#
# dependent environment variables.  To remove this problem, AirsDb.py	#
# function runL1A has been rewritten to avoid the use of ksh scripts at	#
# all.									#
#########################################################################
def runL1A():
   fncName = 'runL1A'

   airs_utils.logstr(fncName, '='*72)
   os.environ['PGS_PC_INFO_FILE'] = PCF_OUT
   Cmd = '%s/PGS_PC_InitCom 1 1 50' %(airs_env.getENV('PGSBIN'))
   airs_utils.logstr(fncName, 'Processing: %s'%(Cmd))
   RESULTS = os.popen(Cmd).readlines()
   airs_utils.loglist(fncName, RESULTS)
   RESULTS=[]

   airs_utils.logstr(fncName, 'Processing: ENVIRONMENT LIST')
   for KEY in os.environ.keys():
      airs_utils.logstr(fncName, '%s\t\t\t%s' % (KEY, os.environ[KEY]))

   airs_utils.logstr(fncName, '='*72)
   
   PthChunk = pathCheck.setPath()
   PthChunk = airs_env.getENV('AIRS_ARCH')+'/bin:'+airs_env.getENV('PGSBIN')+':'+os.environ['PATH']
   airs_utils.logstr(fncName, 'PATH: %s'%(PthChunk))
   os.environ['PATH'] = PthChunk
   
   Cmd = '%s/STORE/pge_scripts/L1A.ksh' % (airs_env.getENV('AIRSROOT'))
   RESULTS = os.popen(Cmd).readlines()
   if (ignore_opt == 0 ):
      for item in RESULTS:
         airs_utils.logstr(fncName, '%s' % (item))
   airs_utils.logstr(fncName, '='*72)

   Cmd = '%s/PGS_PC_TermCom 1 1' % (airs_env.getENV('PGSBIN'))
   airs_utils.logstr(fncName, '='*72)
   airs_utils.logstr(fncName, 'Processing: %s'%(Cmd))
   RESULTS = os.popen(Cmd).readlines()
   airs_utils.loglist(fncName, RESULTS)
   airs_utils.logstr(fncName, '='*72)

   renameFiles.renameFiles(WORKING_DIR, PKT_DIR)
   renameFiles.moveLogs(WORKING_DIR, PKT_DIR)

#########################################################################
def runL1B(PCF_FILE, SCPT):
   fncName = 'runL1B'

   airs_utils.logstr(fncName, '='*72)
   airs_utils.logstr(fncName, 'Processing %s' % (SCPT))
   os.environ['PGS_PC_INFO_FILE'] = PCF_FILE
   Cmd = '%s/PGS_PC_InitCom 1 1 50' %(airs_env.getENV('PGSBIN'))
   airs_utils.logstr(fncName, 'Processing: %s'%(Cmd))
   RESULTS = os.popen(Cmd).readlines()
   airs_utils.loglist(fncName, RESULTS)
   RESULTS=[]

   airs_utils.logstr(fncName, 'Processing: ENVIRONMENT LIST')
   for KEY in os.environ.keys():
      airs_utils.logstr(fncName, '%s\t\t\t%s' % (KEY, os.environ[KEY]))
   airs_utils.logstr(fncName, '='*72)

   PthChunk = pathCheck.setPath()
   PthChunk = airs_env.getENV('AIRS_ARCH')+'/bin:'+airs_env.getENV('AIRSROOT')+'/STORE/pge_scripts:'+airs_env.getENV('PGSBIN')+':'+os.environ['PATH']
   airs_utils.logstr(fncName, 'PATH: %s'%(PthChunk))
   os.environ['PATH'] = PthChunk
   
   Cmd = '%s/STORE/pge_scripts/%s.ksh' % (airs_env.getENV('AIRSROOT'), SCPT)
   RESULTS = os.popen(Cmd).readlines()
   if ( ignore_opt == 0 ):
      for item in RESULTS:
         airs_utils.logstr(fncName, '%s' % (item))
   airs_utils.logstr(fncName, '='*72)

   Cmd = '%s/PGS_PC_TermCom 1 1' % (airs_env.getENV('PGSBIN'))
   airs_utils.logstr(fncName, '='*72)
   airs_utils.logstr(fncName, 'Processing: %s'%(Cmd))
   RESULTS = os.popen(Cmd).readlines()
   airs_utils.loglist(fncName, RESULTS)
   airs_utils.logstr(fncName, '='*72)

   renameFiles.renameFiles(WORKING_DIR, PKT_DIR, 'L1B')
   renameFiles.moveLogs(WORKING_DIR, PKT_DIR)

#########################################################################
def runL2(PCF_FILE,SCPT):
   fncName = 'runL2'

   airs_utils.logstr(fncName, '='*72)
   airs_utils.logstr(fncName, 'Processing %s' % (SCPT))
   os.environ['PGS_PC_INFO_FILE'] = PCF_FILE
   Cmd = '%s/PGS_PC_InitCom 1 1 50' %(airs_env.getENV('PGSBIN'))
   airs_utils.logstr(fncName, 'Processing: %s'%(Cmd))
   RESULTS = os.popen(Cmd).readlines()
   airs_utils.loglist(fncName, RESULTS)
   RESULTS=[]

   airs_utils.logstr(fncName, 'Processing: ENVIRONMENT LIST')
   for KEY in os.environ.keys():
      airs_utils.logstr(fncName, '%s\t\t\t%s' % (KEY, os.environ[KEY]))
   airs_utils.logstr(fncName, '='*72)

   PthChunk = pathCheck.setPath()
   PthChunk = airs_env.getENV('AIRS_ARCH')+'/bin:'+airs_env.getENV('AIRSROOT')+'/STORE/pge_scripts:'+airs_env.getENV('PGSBIN')+':'+os.environ['PATH']
   airs_utils.logstr(fncName, 'PATH: %s'%(PthChunk))
   os.environ['PATH'] = PthChunk

   Cmd = '%s/STORE/pge_scripts/L2.ksh' % (airs_env.getENV('AIRSROOT'))
   RESULTS = os.popen(Cmd).readlines()
   if ( ignore_opt == 0 ) :
      for item in RESULTS:
         airs_utils.logstr(fncName, '%s' % (item))
   airs_utils.logstr(fncName, '='*72)

   Cmd = '%s/PGS_PC_TermCom 1 1' % (airs_env.getENV('PGSBIN'))
   airs_utils.logstr(fncName, '='*72)
   airs_utils.logstr(fncName, 'Processing: %s'%(Cmd))
   RESULTS = os.popen(Cmd).readlines()
   airs_utils.loglist(fncName, RESULTS)
   airs_utils.logstr(fncName, '='*72)

   renameFiles.renameFiles(WORKING_DIR, PKT_DIR, 'L2')
   renameFiles.moveLogs(WORKING_DIR, PKT_DIR)

def usage():
   print ''
   print '-'*72
   print 'AIRS Direct Broadcast System (AIRSDBS):'
   print 'The AIRSDBS processes downloaded raw data packets, EDS,'
   print 'and PDS formatted data packets to Level 1A and Level 1B'
   print 'products.  By default AIRSDBS will attempt to generate'
   print 'both the L1A and L1B outputs.  For specialized needs the'
   print 'system is capable of modified processing, as directed by'
   print 'the following command line options:'
   print ''
   print '-c	: use carryout file in dynamic data directory'
   print '-C	: generate simulated carryout file and use same.'
   print '-p	: generate only PCF files; don\'t execute'
   print '-a	: generate both L1A & LB PCF files; execute L1A only.'
   print '-l string  : generate the output products at below levels:'
   print '             0  = no output products (generate only pcf files)'
   print '             1A = level 1A only, '
   print '             1B = up to level 1B, '
   print '             2  = up to level 2,'
   print '             default output product level is 1B'
   print '-u    : do not compress L1B AIRS or L2 products'
   print '             default option is compressing L1A AIRS and L2 products'
   print '-f    : do regression-only retrieval'
   print '              default option is defined in L2.pcf template(LID 1005)' 
   print '-t directory : use the leapsec.dat and utcpole.dat at this'
   print '               directory. The default is using the toolkit directory'
   print '-d #  : set the print level IO and processing. (0=off, 1=low,'
   print '          2=medium, 3=high) default level value is 0 '
   print '-v    : print debug statement to screen.'
   print '-i    : do not save the PGE debug messages to log file.'
   print '-r    : do not cleanup the temporary files.'
   print '-D string  : Start Date in the format of YYYY-MM-DD'
   print '-G #  : Start Granule number (between 1 and 240)'
   print '-N #  : Number of Granules to be processed'
   print '-h	: print usage message and exit.'
   print ''
   print 'NOTE: the -o option, which generate and use simulated '
   print 'Attitude and Ephemeris files for this run, is no longer supported'
   print ''
   print 'NOTE: if Attitude and Ephemeris files are found in the'
   print 'directory where the downloaded data packets are stored,'
   print 'AIRSDBS will utilize them in processing.  Not having these'
   print 'files is NOT an error, however processing times may increase'
   print 'without them.'
   print '-'*72
   sys.exit(0)

#########################################################################
# main process control.							#
#########################################################################
#########################################################################
# command line parameters are processed here.  Note that the design	#
# requires that normal operations will proceed from the command line	#
# alone, without command line options.  The options available are:	#
#	-c	: use carryout file in dynamic data directory		#
#	-C	: generate simulated carryout file and use same.	#
#	-p	: generate only PCF files; don't execute		#
#	-a	: generate both L1A & LB PCF files; execute L1A only.	#
#       -l string  : generate the output products at below levels:'     #
#                       0  = no output products(generate only pcf files)#
#                       1A = level 1A only                              #
#                       1B = up to level 1B                             #
#                       2  = up to level 2                              #
#                       default output product level is 1B              #
#       -u      : do not compress L1B AIRS or L2 products               #
#                       default option is compressing L1A AIRS or L2    #
#       -f      : do regression-only retrieval                          #
#                       default option is defined in L2.pcf template    #
#       -t directory   : use the leapsec.dat and utcpole.dat at this    #
#                        directory. The default is using the toolkit    #
#                        directory                                      #
#       -d #    : set the print level IO and processing. (0=off, 1=low, #
#                   2=medium, 3=high) default level value is 0          #
#       -i      : do not save the PGE debug messages to log file.       #
#       -r      : do not cleanup the temporary files.                   #
#       -D string  : Start Date in the format of YYYY-MM-DD             #
#       -G #    : Start Granule number (between 1 and 240)              #
#       -N #    : Number of Granules to be processed                    #
#       -v      : print debug statement to screen.                      #
#	-h	: print usage message and exit.				#
# Values based on these options are set in airs_env module and accessed	#
# from there.								#
#########################################################################

# A problem was identified when the user tries to run in the actual source
# directory where the data files are located, and sets AIRSDB_PACKETS to
# '.'  We need to check for this and reset AIRSDB_PACKETS to $PWD.
fncName='AirsDb.py-main'

DateStamp = getDateStr()
TimeStamp = DateStamp[2:7]+DateStamp[8:]

LogFileName = 'AIRSDB_LOG-%s.log'%(TimeStamp)
airs_utils.setlog(os.getcwd()+'/'+LogFileName)

PKT_DIR = os.environ['AIRSDB_PACKETS']
airs_utils.logstr(fncName, 'PKT_DIR = %s' %(PKT_DIR))
if (PKT_DIR == '.'):
   os.environ['AIRSDB_PACKETS'] = os.environ['PWD']
   PKT_DIR = os.environ['AIRSDB_PACKETS']
airs_utils.logstr(fncName, 'actual PKT_DIR = %s' %(PKT_DIR))

# Initialize the airs environment variable table.
airs_env.initENVTbl()
airs_env.initIndexTbl()
TEMP = airs_env.getENV('AIRSROOT')+'/STORE/ancillary'
airs_env.setENV('ANC_STATIC', TEMP)
TEMP = airs_env.getENV('AIRSDB_DYNAMIC')
airs_env.setENV('ANC_DYNAMIC', TEMP)
TEMP = airs_env.getENV('AIRSDB_RUN')
airs_env.setENV('WORKING_DIR', TEMP)
TEMP = airs_env.getENV('AIRSROOT')+'/STORE/mcf'
airs_env.setENV('MCF_DIR', TEMP)
TEMP = airs_env.getENV('AIRSROOT')+'/STORE/pcf'
airs_env.setENV('PCF_DIR', TEMP)

airs_env.setENV('GEN_ATT_EPHEM_FILES', 0)
airs_env.setENV('PCF_ONLY', 0)
airs_env.setENV('CARRYOUT', 0)
airs_env.setENV('GEN_CARRYOUT', 0)
airs_env.validateENV()
airs_env.setPath()

#airs_utils.logstr(fncName, 'PCF_OUT=%s'%(PCF_OUT))

# NOTE: opts is a list of tuples, two items per tuple.  opts[N][0] is
# the option, opts[N][1] is the associated value.
# now process command line options.
opts, args = getopt.getopt(sys.argv[1:], "cCopahl:uft:d:virmD:G:N:")
for option in opts:
  if (option[0]=='-h'):
     usage()
  if (option[0]=='-c'):
     airs_env.setENV('CARRYOUT', 1)
  if (option[0]=='-C'):
     airs_env.setENV('CARRYOUT', 1)
     airs_env.setENV('GEN_CARRYOUT', 1)
  if (option[0]=='-o'):
     usage()
     #airs_env.setENV('GEN_ATT_EPHEM_FILES', 1)
  if (option[0]=='-p'):
     pcf_opt = 1 
  if (option[0]=='-a'):
     proc_opt = '1a'
  if (option[0]=='-l'):
     proc_opt = option[1]
  if (option[0]=='-u'):
     uncompress_opt = 'UnCompressed' 
  if (option[0]=='-f'):
     runl2mode_opt = 3 
  if (option[0]=='-t'):
     dir_utc_leap_opt = option[1]
  if (option[0]=='-d'):
     print_opt = string.atoi(option[1]) % 4
  if (option[0]=='-v'):
     debug_opt = 1
  if (option[0]=='-i'):
     ignore_opt = 1
  if (option[0]=='-r'):
     remove_opt = 1
  if (option[0]=='-m'):
     gen_MET_opt = 1
  if (option[0]=='-D'):
      Override_ST_Date = option[1]
  if (option[0]=='-G'):
      Override_ST_Granule = string.atoi(option[1])
  if (option[0]=='-N'):
      Override_Num_Granule = string.atoi(option[1])

printDEBUG('Override ST_Date = %s' % Override_ST_Date)
printDEBUG('Override ST_Granule = %d'% Override_ST_Granule)
printDEBUG('Override Num_Granule = %d' % Override_Num_Granule)
printDEBUG('Process Trace level = %d' % print_opt)
printDEBUG('Processing output product level = %s' % proc_opt)
printDEBUG('Product Compression level = %s' % uncompress_opt)

print "Info: Processing Trace level = %d" % print_opt
print "Info: Processing output product level = %s" % proc_opt
print "Info: Product Compression level = %s" % uncompress_opt

# Set the processing level
if (pcf_opt == 1 or proc_opt == '0'):
   airs_env.setENV('EXEC_L1A',0)
   airs_env.setENV('EXEC_L1B',0)
   airs_env.setENV('EXEC_L2' ,0)
elif (proc_opt == '1a' or proc_opt == '1A'):
   airs_env.setENV('EXEC_L1A',1)
   airs_env.setENV('EXEC_L1B',0)
   airs_env.setENV('EXEC_L2' ,0)
elif (proc_opt == '1b' or proc_opt == '1B'):
   airs_env.setENV('EXEC_L1A',1)
   airs_env.setENV('EXEC_L1B',1)
   airs_env.setENV('EXEC_L2' ,0)
elif (proc_opt == '2'):
   airs_env.setENV('EXEC_L1A',1)
   airs_env.setENV('EXEC_L1B',1)
   airs_env.setENV('EXEC_L2' ,1)
else:
   airs_env.setENV('EXEC_L1A',0)
   airs_env.setENV('EXEC_L1B',0)
   airs_env.setENV('EXEC_L2' ,0)

# Now check to see what type of raw data files we have - PDS, EDS, or
# true raw/EDU packets.  These will be located where AIRSDB_PACKETS
# points.  If raw/EDU data, then generate PDS to WORKING_DIR, else
# generate links in WORKING_DIR to the L0 PDS/EDS in AIRSDB_PACKETS

# Set working directory.
WORKING_DIR = os.path.join(airs_env.getENV('AIRSDB_RUN'),DateStamp)
# Set the metadata template path/name for current envirionment.
METATEMPLATE = os.path.join(airs_env.getENV('AIRSDB'),'data',METATEMPLATE)
os.mkdir(WORKING_DIR)
airs_env.setENV('WORKING_DIR', WORKING_DIR)
airs_env.setENV('CURRENT_DIR', WORKING_DIR)
esdt.gen_esdt_tbl()

# Set the architecture based on the environment variable ARCH
airs_utils.setArch()

# Reset the dummy pcf files for using the leapsec.dat and utcpole.dat
L1A_PCF = '%s/STORE/pcf/L1A-AIRS.pcf' % (airs_env.getENV('AIRSROOT'))
UTIL_PCF = WORKING_DIR+'/Util.pcf'
CMD_cp = '/bin/cp %s %s ;/bin/chmod +w %s' % (L1A_PCF,UTIL_PCF,UTIL_PCF)
Result_cp=os.popen( CMD_cp ).readline()[:1]
if ( dir_utc_leap_opt != ""):
   UTIL = pcf_template.PCF(UTIL_PCF)
   repLIDLine(10301,UTIL,dir_utc_leap_opt,'leapsec.dat')
   repLIDLine(10401,UTIL,dir_utc_leap_opt,'utcpole.dat')
   UTIL.writePCF(UTIL_PCF)
   

os.environ['PGS_PC_INFO_FILE']= UTIL_PCF
L1A_INSTRUMENTS=['AMSU', 'AIRS', 'HSB', 'VIS']
InstGrps = genL0()

for Instrument in InstGrps:
   ST_Index = 'STARTTIME-%s' % (Instrument)
   ET_Index = 'ENDTIME-%s' % (Instrument)
   ST_Str = airs_env.getENV(ST_Index)
   ET_Str = airs_env.getENV(ET_Index)

   airs_env.setENV('GRANULE_SIZE',  '45')
   airs_env.setENV('FIRST_GRANULE', `first_gr`)
   airs_env.setENV('LAST_GRANULE',  `last_gr`)
   airs_env.setENV('GRANULE_COUNT', `num_gr`)

   airs_env.setENV('INSTRUMENT', `L1A_INSTRUMENTS.index(Instrument)`)
   TMPSTR = 'LogStatus-%s-%s' %('L1A',Instrument)
   airs_env.setENV('LOGSTATUS', TMPSTR)
   TMPSTR = 'LogReport-%s-%s' %('L1A',Instrument)
   airs_env.setENV('LOGREPORT', TMPSTR)
   TMPSTR = 'LogUser-%s-%s' %('L1A',Instrument)
   airs_env.setENV('LOGUSER', TMPSTR)
   TMPSTR = 'ShmMem-%s-%s' %('L1A',Instrument)
   airs_env.setENV('SHMMEM', TMPSTR)
   TMPSTR=''

   PCF_OUT = os.path.join(WORKING_DIR, apid.getInstrumentPCFTemplate(Instrument, 'L1A'))
   PCF_IN  = os.path.join(airs_env.getENV('PCF_DIR'), apid.getInstrumentPCFTemplate(Instrument, 'L1A'))
   airs_utils.logstr(fncName, 'PCF_IN=%s'%(PCF_IN))

   PCF_TBL = pcf_template.PCF(PCF_IN)
   PCF_TBL.setRepTblVal('sgi64', airs_utils.getArch())
   PCF_TBL.setRepTblVal('must-set-mcf-directory', airs_env.ENV['MCF_DIR'])
   PCF_TBL.setRepTblVal('must-set-ancillary-dir', airs_env.ENV['ANC_STATIC'])
   PCF_TBL.setRepTblVal('must-set-dynamic-directory', airs_env.ENV['ANC_DYNAMIC'])
   PCF_TBL.setRepTblVal('must-set-current-directory', airs_env.ENV['WORKING_DIR'])

   PCF_TBL.setInstrument(Instrument)
   PCF_TBL.setValidAPIDSet()
   PCF_TBL.genAPIDDataSet(WORKING_DIR)

   # Set Print Level IO and Processing
   PCF_TBL.replaceField('1002','%d' % print_opt, 2)
   PCF_TBL.replaceField('1003','%d' % print_opt, 2)
   # Set processing facility to 'D' for "Direct Broadcast"
   PCF_TBL.replaceField('1021', 'D', 2)
   # Set the utcpole.dat and leapsec.dat if required
   if (dir_utc_leap_opt != "" ) :
      repLIDLine(10301,PCF_TBL,dir_utc_leap_opt,'leapsec.dat')
      repLIDLine(10401,PCF_TBL,dir_utc_leap_opt,'utcpole.dat')
      

   if (airs_env.getENV('CARRYOUT')):
      proc_CO_Files(PKT_DIR)

   if (airs_env.ENV.has_key('GEN_ATT_EPHEM_FILES')):
      if (airs_env.getENV('GEN_ATT_EPHEM_FILES')!=1):
         WRK_DIR = airs_env.getENV('WORKING_DIR')
         ATTLIST, EPHLIST = getAtt_n_Ephem_Files(WRK_DIR)
      else:
         ATTLIST, EPHLIST = getAtt_n_Ephem_Files()

      PCF_TBL.expandMultiList(10502, ATTLIST)
      PCF_TBL.expandMultiList(10501, EPHLIST)

   PCF_TBL.doReplacements()

   for key in (airs_env.IndexTbl.keys()):
      x = 0
      for x in range (len(airs_env.IndexTbl[key])):
         IdxSet = airs_env.getIndex(key, x)
	 if (airs_env.getENV(key) != 'UNKNOWN'):
            PCF_TBL.replaceField(IdxSet[0], airs_env.getENV(key), IdxSet[1])
   expandOutputEntries(airs_env.getENV('GRANULE_COUNT'),
                       airs_env.getENV('FIRST_GRANULE'),
		       airs_env.getENV('LAST_GRANULE'), PCF_TBL, 'L1A')
   PCF_TBL.setMetafiles()
   PCF_TBL.writePCF(PCF_OUT)

   if (airs_env.getENV('EXEC_L1A') == 1):
      runL1A()

# Now process L1B
airs_utils.loglist(fncName, L1B_INPUT_SETS.keys(), 'L1B_INPUT_SETS:')

# NOTE: L1A_AIRS processing leads to L1B-AIRS and L1B-VIS input sets.
# What is needed first is to determine the instrument sets that must be
# processed for L1B

L1B_Instruments=[]
for idx in L1B_INPUT_SETS.keys():
   Inst = apid.getInstrumentName(`idx`)
   airs_utils.logstr(fncName, '%s, %s' %(Inst, L1B_INPUT_SETS[idx]))
   if not (Inst in L1B_Instruments):
      L1B_Instruments.append(Inst)

airs_utils.loglist(fncName,L1B_Instruments,'L1B_Instruments')

for Inst in L1B_Instruments:
   airs_utils.logstr(fncName, 'L1B-%s Processing' %(Inst))
   airs_env.setENV('INSTRUMENT', `apid.getInstVal(Inst)`)
   TMPSTR = 'LogStatus-%s-%s' %('L1B',Inst)
   airs_env.setENV('LOGSTATUS', TMPSTR)
   TMPSTR = 'LogReport-%s-%s' %('L1B',Inst)
   airs_env.setENV('LOGREPORT', TMPSTR)
   TMPSTR = 'LogUser-%s-%s' %('L1B',Inst)
   airs_env.setENV('LOGUSER', TMPSTR)
   TMPSTR = 'ShmMem-%s-%s' %('L1B',Inst)
   airs_env.setENV('SHMMEM', TMPSTR)
   TMPSTR=''
   SCRIPT_FLAG='L1B_%s' % (Inst)
   PCF_OUT = os.path.join(WORKING_DIR,
   			  apid.getInstrumentPCFTemplate(Inst, 'L1B'))
   PCF  = os.path.join(airs_env.getENV('PCF_DIR'),
   		       apid.getInstrumentPCFTemplate(Inst, 'L1B'))
   airs_utils.logstr(fncName, 'L1B-%s PCF Template: %s' %(Inst,PCF))
   L1B_PCF = pcf_template.PCF(PCF)
   L1B_PCF.setRepTblVal('sgi64', airs_utils.getArch())
   L1B_PCF.setRepTblVal('must-set-mcf-directory',
   			airs_env.ENV['MCF_DIR'])
   L1B_PCF.setRepTblVal('must-set-ancillary-dir',
   			airs_env.ENV['ANC_STATIC'])
   L1B_PCF.setRepTblVal('must-set-dynamic-directory',
   			airs_env.ENV['ANC_DYNAMIC'])
   L1B_PCF.setRepTblVal('must-set-current-directory',
   			airs_env.ENV['WORKING_DIR'])

   L1B_PCF.setInstrument(Inst)
   L1B_PCF.setValidAPIDSet()
   # Set print level IO and processing
   L1B_PCF.replaceField('1002', '%d' % print_opt, 2)
   L1B_PCF.replaceField('1003', '%d' % print_opt, 2)
   # Set processing facility to 'D' for "Direct Broadcast"
   L1B_PCF.replaceField('1021', 'D', 2)

   # Set LID 1071 "Processing Directives" to Nominal or UnCompressed
   if (Inst == 'AIRS') :
      L1B_PCF.replaceField('1071', uncompress_opt, 2)

   # Set the utcpole.dat and leapsec.dat if required
   if (dir_utc_leap_opt != "" ) :
      repLIDLine(10301,L1B_PCF,dir_utc_leap_opt,'leapsec.dat')
      repLIDLine(10401,L1B_PCF,dir_utc_leap_opt,'utcpole.dat')

   if (airs_env.ENV.has_key('GEN_ATT_EPHEM_FILES')):
      L1B_PCF.expandMultiList(10502, ATTLIST)
      L1B_PCF.expandMultiList(10501, EPHLIST)

   L1B_PCF.doReplacements()
   airs_utils.logstr(fncName, 'calling expandOutputEntries')
   expandOutputEntries(airs_env.getENV('GRANULE_COUNT'),
                       airs_env.getENV('FIRST_GRANULE'),
		       airs_env.getENV('LAST_GRANULE'), L1B_PCF, 'L1B')
   airs_utils.logstr(fncName, 'post expandOutputEntries')
   for LID in L1B_INPUT_SETS.keys():
      ChkInst = apid.getInstrumentName(`LID`)
      airs_utils.logstr(fncName, '<%d> <%s> == <%s>' % (LID, ChkInst, Inst))
      if (ChkInst == Inst):
	 #L1B_PCF.clearCommentFlag(`LID`)
         L1B_PCF.expandMultiList(LID, L1B_INPUT_SETS[LID])
   for key in (airs_env.IndexTbl.keys()):
      x = 0
      airs_utils.logstr(fncName, 'key = %s'%(key))
      for x in range (len(airs_env.IndexTbl[key])):
         IdxSet = airs_env.getIndex(key, x)
	 if (airs_env.getENV(key) != 'UNKNOWN'):
	    airs_utils.logstr(fncName, '%s'%(IdxSet[0]))
            airs_utils.logstr(fncName, 'triplet = <%s-%s-%s>'%(IdxSet[0], airs_env.getENV(key), IdxSet[1]))
            L1B_PCF.replaceField(IdxSet[0], airs_env.getENV(key), IdxSet[1])
   L1B_PCF.writePCF(PCF_OUT)
   if (airs_env.getENV('EXEC_L1B') == 1):
      runL1B(PCF_OUT, SCRIPT_FLAG)

# Now process L2
#airs_utils.loglist(fncName, L2_INPUT_SETS.keys(), 'L2_INPUT_SETS:')

L2_Instruments=L1B_Instruments

airs_utils.loglist(fncName,L2_Instruments,'L2_Instruments')

printDEBUG(" first granule = %d, last granule = %d"%(first_gr,last_gr))
ST_granule=ST_tm
ET_granule=ST_granule+359.0
for inum in range (0,num_gr):
    Igranule = (first_gr+inum-1)%240+1;
    printDEBUG("L2 processing:  at granule %d"%Igranule)

    ST_UTC = toUTC(ST_granule)
    ET_UTC = toUTC(ET_granule)
    printDEBUG( "  ST_UTC = %s  ST_granule =%d"%(ST_UTC,ST_granule))
    printDEBUG( "  ET_UTC = %s  ET_granule =%d"%(ET_UTC,ET_granule))
    printDEBUG( "  time difference= %d"%(ET_granule-ST_granule))

    Inst = 'AIRS'

    airs_env.setENV('STARTTIME-AIRS', ST_UTC)
    airs_env.setENV('ENDTIME-AIRS', ET_UTC)
    airs_env.setENV('STARTTIME-AMSU', ST_UTC)
    airs_env.setENV('ENDTIME-AMSU', ET_UTC)
    airs_env.setENV('STARTTIME-HSB', ST_UTC)
    airs_env.setENV('ENDTIME-HSB', ET_UTC)

    airs_utils.logstr(fncName, 'L2-%d Processing' %(Igranule))
    airs_env.setENV('INSTRUMENT', `apid.getInstVal(Inst)`)
    TMPSTR = 'LogStatus-%s-%d' %('L2',Igranule)
    airs_env.setENV('LOGSTATUS', TMPSTR)
    TMPSTR = 'LogReport-%s-%d' %('L2',Igranule)
    airs_env.setENV('LOGREPORT', TMPSTR)
    TMPSTR = 'LogUser-%s-%d' %('L2',Igranule)
    airs_env.setENV('LOGUSER', TMPSTR)
    TMPSTR = 'ShmMem-%s-%d' %('L2',Igranule)
    airs_env.setENV('SHMMEM', TMPSTR)

    airs_env.setENV('GRANULE_SIZE',  '45')
    airs_env.setENV('FIRST_GRANULE', `Igranule`)
    airs_env.setENV('LAST_GRANULE',  `Igranule`)
    airs_env.setENV('GRANULE_COUNT', `1`)

    SCRIPT_FLAG='L2'
    PCF_OUT = os.path.join(WORKING_DIR, 'L2-%d.pcf'%Igranule)
    PCF  = os.path.join(airs_env.getENV('PCF_DIR'),
                    apid.getInstrumentPCFTemplate('AIRS', 'L2'))
    airs_utils.logstr(fncName, 'L2-%d PCF Template: %s'%(Igranule,PCF))
    L2_PCF = pcf_template.PCF(PCF)
    L2_PCF.setRepTblVal('sgi64', airs_utils.getArch())
    L2_PCF.setRepTblVal('must-set-mcf-directory',
                         airs_env.ENV['MCF_DIR'])
    L2_PCF.setRepTblVal('must-set-ancillary-dir',
                     airs_env.ENV['ANC_STATIC'])
    L2_PCF.setRepTblVal('must-set-dynamic-directory',
                        airs_env.ENV['ANC_DYNAMIC'])
    L2_PCF.setRepTblVal('must-set-current-directory',
                        airs_env.ENV['WORKING_DIR'])

    L2_PCF.setInstrument(Inst)
    L2_PCF.setValidAPIDSet()
    if (airs_env.ENV.has_key('GEN_ATT_EPHEM_FILES')):
       L2_PCF.expandMultiList(10502, ATTLIST)
       L2_PCF.expandMultiList(10501, EPHLIST)

    # Setup the available GRIB files 
    getGRIB_Files()

    # Set print level IO and processing
    L2_PCF.replaceField('1002', '%d' % print_opt, 2)
    L2_PCF.replaceField('1003', '%d' % print_opt, 2)
    # Set the run level 2 mode option
    if (runl2mode_opt > 0) :
       L2_PCF.replaceField('1005', '%d' % runl2mode_opt, 2)
    # set the statistic mode to zero
    L2_PCF.replaceField('1006','0',2)
    # Set processing facility to 'D' for "Direct Broadcast"
    L2_PCF.replaceField('1021', 'D', 2)
    # Correct some fields in L2 PCF file
    # set Number of Granules to process
    L2_PCF.replaceField('1022','1',2)  
    # set start scan within granule
    L2_PCF.replaceField('1017','1',2)  
    # Set LID 1071 "Processing Directives" to Nominal or UnCompressed
    Proc_directive = uncompress_opt
    if (GRIB_IDX > 0) :
       Proc_directive = uncompress_opt + ':GribForecast'
    L2_PCF.replaceField('1071', Proc_directive, 2)
    # Set the utcpole.dat and leapsec.dat if required
    if (dir_utc_leap_opt != "" ) :
       repLIDLine(10301,L2_PCF,dir_utc_leap_opt,'leapsec.dat')
       repLIDLine(10401,L2_PCF,dir_utc_leap_opt,'utcpole.dat')
    # set the start/end UTC time for each granule
    L2_PCF.replaceField('10258',toUTC(ST_granule),2) 
    L2_PCF.replaceField('10259',toUTC(ET_granule),2)  
      

    L2_PCF.doReplacements()

    airs_utils.logstr(fncName, 'calling expandOutputEntries')
    expandOutputL2(Igranule,L2_PCF)

    airs_utils.logstr(fncName, 'post expandOutputEntries')
    expandInputL2(Igranule,L2_PCF)

    for key in (airs_env.IndexTbl.keys()):
       x = 0
       airs_utils.logstr(fncName, 'key = %s'%(key))
       for x in range (len(airs_env.IndexTbl[key])):
          IdxSet = airs_env.getIndex(key, x)
          if (airs_env.getENV(key) != 'UNKNOWN'):
             airs_utils.logstr(fncName, '%s'%(IdxSet[0]))
             airs_utils.logstr(fncName, 'triplet = <%s-%s-%s>'%(IdxSet[0], airs_env.getENV(key), IdxSet[1]))
             L2_PCF.replaceField(IdxSet[0], airs_env.getENV(key), IdxSet[1])


    L2_PCF.setMetafiles()
    # Write out L2 PCF file
    L2_PCF.writePCF(PCF_OUT)

    # Update the granule
    ST_granule = ST_granule + 360.0
    ET_granule = ET_granule + 360.0

    if (airs_env.getENV('EXEC_L2') == 1):
       runL2(PCF_OUT, SCRIPT_FLAG)

# Now clean up the original output files with bogus names:
if (remove_opt == 0) : airs_utils.cleanup(WORKING_DIR)
